#include <Array.au3>
#include <File.au3>

; Script constants
Global Const $nWinTimeout = 3
Global Const $nAppTimeout = 5
Global Const $nGWWinTimeout  = 3

; Autologin for Guild Wars
Global Enum $idEMail, $idPassword, $idCharacter, $_sizeCommands
Global $arrCommandLines[$_sizeCommands] = [ "email", "password", "character" ]
Global $arrLoginParams[3] = [ "", "", "" ]
Global $bGotLoginParams = False

; Script main parameters
Global Const $strIniPath = GetIniPath()
Global Const $strTexPath = IniReadError($strIniPath, "Settings", "TexModPath")
Global Const $strAppPath = IniReadError($strIniPath, "Settings", "AppPath")
Global $arrMods[1] = [""]
Global Const $strPostrunPath = IniRead($strIniPath, "Settings", "PostrunPath", "")
Global Const $bSplash = IniRead($strIniPath, "Settings", "Splash", "1")

main()
Func main()
	LoadCommandLines()
	LoadModList()
	
	Local $strSplashText = "Starting Texmod..."
	UpdateSplash($strSplashText)
	
	; If any application instances are stuck in background, kill them
	KillTexmod()
	
	; Run TexMod
	If Not Run($strTexPath, "", @SW_HIDE) Then
		ShowError("Can't start texmod!")
		Exit
	EndIf
	If Not WinWait("TexMod", "", $nWinTimeout) Then
		ShowError("Timeout: Texmod didn't open. Please restart AutoTexmod!")
		Exit
	EndIf
	WinMove("TexMod", "", 0, 0)

	$strSplashText &= @CRLF & "Selecting the application..."
	UpdateSplash($strSplashText)

	; Select the application
	LoadFileWithBrowse(200, "Select Executable", $strAppPath)

	$strSplashText &= @CRLF & "Loading the modifications..."
	UpdateSplash($strSplashText)

	; Load the Texmod mods
	For $strMod In $arrMods
		If FileExists($strMod) Then
			LoadFileWithBrowse(500, "Select Texmod Packages to add.", $strMod)
		EndIf
	Next

	$strSplashText &= @CRLF & "Starting the application..."
	UpdateSplash($strSplashText)

	; Start the application
	ControlClick("TexMod", "", "[CLASS:Button; INSTANCE:11]")

	; Wait for the process to start, option to kill texmod if it doesn't
	WaitForApp()

	; Exit TexMod
	WinClose("TexMod")

	; Start the "postrun" application
	If FileExists($strPostrunPath) Then
		Run($strPostrunPath)
	EndIf

	SplashOff()

	; Auto login for Guild Wars
	AutoLogin()

	; Clear up any temporary files not currently used
	ClearTemp()
EndFunc ;==> main

;;;;;;;;;;;;;;;;;;;;;;;;;
; Configuration functions

; Create an initial configuration file
Func MakeConfigFile($strFile)
	; Ask the user to create a config file
	If MsgBox(4, "AutoTexmod Error", "Cannot find the AutoTexmod configuration file." & @CRLF & "Would you like to create a new one?") == 7 Then
		Return False
	EndIf
	
	; Try if a file can be created
	If Not _FileCreate($strFile) Then
		ShowError("Cannot create configuration file!")
	EndIf

	Local $strBuffer
	
	; Get the Texmod path
	If FileExists(@ScriptDir & "Texmod.exe") Then
		$strBuffer = @ScriptDir & "Texmod.exe"
	Else
		$strBuffer = FileOpenDialog("Please select Texmod.exe!", @ScriptDir, "Texmod (Texmod.exe)", 1)
		If @error Then
			FileDelete($strFile)
			Return False
		EndIf
	EndIf
	IniWrite($strFile, "Settings", "TexModPath", $strBuffer)

	; Get the application path
	$strBuffer = FileOpenDialog("Please select the application that you want to use the modifications on!", "", "Executables (*.exe)", 1)
	If @error Then
		FileDelete($strFile)
		Return False
	EndIf
	IniWrite($strFile, "Settings", "AppPath", $strBuffer)
	Local $strAppPath = $strBuffer ; Saving the application path for shortcut generation here
	
	; Get the modifications
	$strBuffer = FileOpenDialog("Please select a modification to load! (Click Cancel when done)", "", "Modification (*.tpf)", 1)
	If @error Then
		FileDelete($strFile)
		Return False
	EndIf
	Local $i = 1
	While Not @error
		IniWrite($strFile, "Mods", "Mod" & $i, $strBuffer)
		$i += 1
		
		$strBuffer = FileOpenDialog("Please select a modification to load! (Click Cancel when done)", "", "Modification (*.tpf)", 1)
	WEnd

	MsgBox(0, "AutoTexmod Information", "Configuration file written to:" & @CRLF & $strFile & @CRLF & @CRLF & "If you want to rerun this setup again, please delete this file!")
	
	; Create shortcut
	If MsgBox(4, "AutoTexmod Information", "Would you like to create a shortcut on your desktop for starting your application with your mods?") Then
		$strBuffer = ""
		If (FileGetVersion($strAppPath, "ProductName") == "Guild Wars") And (MsgBox(4, "AutoTexmod Configuration", "Guild Wars detected. Would you like to set automatic login information?") == 6) Then
			$bGotLoginParams = True
			Local $strInput
			$strInput = InputBox("AutoTexmod Configuration", "Please enter your e-mail address (optional)")
			If $strInput Then
			   $strBuffer &= "-email """ & $strInput & """ "
			   $arrLoginParams[$idEMail] = $strInput
			EndIf
			$strInput = InputBox("AutoTexmod Configuration", "Please enter your password (needed for auto-login)", "", "*")
			If $strInput Then
			   $strBuffer &= "-password """ & $strInput & """ "
			   $arrLoginParams[$idPassword] = $strInput
			EndIf
			$strInput = InputBox("AutoTexmod Configuration", "Please enter one of your characters' name (optional)")
			If $strInput Then
			   $strBuffer &= "-character """ & $strInput & """ "
			   $arrLoginParams[$idCharacter] = $strInput
			EndIf
		EndIf
		Local $strAppName = GuessAppName($strAppPath)
		FileCreateShortcut(@ScriptFullPath, _											; Shortcut target
		                   @DesktopDir & "\" & $strAppName & ".lnk", _					; Shortcut path
						   "", _														; Working directory
						   StringStripWS($strBuffer, 2), _								; Arguments (passed only for GW)
						   $strAppName, _												; Shortcut description
						   $strAppPath)													; Used icon file (using icon index 0)
	EndIf

	Return True
EndFunc ;==> MakeConfigFile

; Returns the ini file path
Func GetIniPath()
	If FileExists(@MyDocumentsDir & "\Guild Wars\AutoTexmod.ini") Then
		Return @MyDocumentsDir & "\Guild Wars\AutoTexmod.ini"
	ElseIf FileExists(@MyDocumentsDir & "\Guild Wars\Mods.ini") Then
		Return @MyDocumentsDir & "\Guild Wars\Mods.ini"
	ElseIf FileExists(@ScriptDir & "\AutoTexmod.ini") Or MakeConfigFile(@ScriptDir & "\AutoTexmod.ini") Then
		Return @ScriptDir & "\AutoTexmod.ini"
	Else		
		ShowError("Cannot find or create the texmod configuration file!", "AutoTexmod Configuration Error")
		Exit
	EndIf
EndFunc ;==> GetIniPath

; Ini reader with error messaging
Func IniReadError($strIniPath, $strSection, $strVariable, $bExistNeed = True)
	Local $strReturn = IniRead($strIniPath, $strSection, $strVariable, "###")
	If $strReturn == "###" Then
		If $bExistNeed Then
			ShowError("Cannot fine the ini value for """ & $strVariable & """!", "AutoTexmod Configuration Error")
			Exit
		Else
			Return ""
		EndIf
	EndIf
	If Not FileExists($strReturn) Then
		ShowError("Cannot fine the file """ & $strReturn & """ in the ini setting """ & $strVariable & """!", "AutoTexmod Configuration Error")
		Exit
	EndIf
	
	Return $strReturn
EndFunc ;==> IniReadError

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Texmod controlling functions

; Function to load the list of mods
Func LoadModList()
	Local $strBuffer
	Local $i = 1
		
	$strBuffer = IniReadError($strIniPath, "Mods", "Mod" & $i, False)
	While ($strBuffer <> "")
		_ArrayAdd($arrMods, $strBuffer)
		
		$i = $i + 1
		$strBuffer = IniReadError($strIniPath, "Mods", "Mod" & $i, False)
	WEnd
EndFunc ;==> LoadModList

; Function that makes sure the browse window for the application is up
Func LoadFileWithBrowse($idCommand, $strBrowseTitle, $strFilePath)
	; Get the browse window by window command
	If Not PostCommandMessage("TexMod", "", $idCommand) Then
		ShowError("Cannot send popup command to the window")
		WinClose("TexMod")
		WinKill("TexMod")
		Exit
	EndIf

	; Wait for the browse window to open
	If Not WinWait($strBrowseTitle, "", $nWinTimeout) Then
		ShowError("Timeout: A browse for file window didn't open. Please restart AutoTexmod!")
		WinClose("TexMod")
		WinKill("TexMod")
		Exit
	EndIf
	
	; Enter the opened file's path into the text field of the browse window
	ControlSetText($strBrowseTitle, "", "[CLASS:Edit; INSTANCE:1]", $strFilePath)
	
	; Click the open button of the browse window
	Local $idStyle = ControlGetStyle($strBrowseTitle, "", "[CLASS:Button; INSTANCE:1]")
	If @error Or BitAND($idStyle, 0x2) Then ; Checking if the button style includes BS_CHECKBOX == 0x2
		ControlClick($strBrowseTitle, "", "[CLASS:Button; INSTANCE:2]")
	Else
		ControlClick($strBrowseTitle, "", "[CLASS:Button; INSTANCE:1]")
	EndIf
EndFunc ;==> GetBrowseWindow

;;;;;;;;;;;;;;;;;;;
; Display functions

; Update (or display) the splash screen with the given text
Func UpdateSplash($strText)
	If $bSplash <> "1" Then
		Return
	EndIf
	If Not ControlSetText("Starting the application", "", "Static1", $strText & @CRLF & "Please wait...") Then
		SplashTextOn("Starting the application", $strText & @CRLF & "Please wait...", 350, 120)
	EndIf	
EndFunc ;==> UpdateSplash

; Show an error dialog
Func ShowError($strError, $strErrorTitle = "AutoTexmod Error")
	SplashOff()
	MsgBox(0, $strErrorTitle, $strError)
EndFunc ;==> ShowError

;;;;;;;;;;;;;;;;;;;;;;;;
; Miscelaneous functions

; Kill any texmod or autotexmod instances
Func KillTexmod()
	Local $strTexName = FileGetName($strTexPath)
	While ProcessExists($strTexName)
		ProcessClose($strTexName)
	WEnd
EndFunc ;==> KillTexmod

; Wait for the application to start otherwise ask to kill texmod if it doesn't
Func WaitForApp()
	Local $strAppName = FileGetName($strAppPath)
	While Not ProcessWait($strAppName, $nAppTimeout)
		SplashOff()
		If MsgBox(4, "AutoTexmod error", "Timeout: Texmod hasn't started the application. Would you like to forcibly close Texmod?") == 6 Then
			WinClose("TexMod")
			WinKill("TexMod")
			Exit
		EndIf
		UpdateSplash("Starting the application...")
	WEnd
EndFunc ;==> WaitForApp

; Clear temporary files
Func ClearTemp()
	Sleep(500)
	
	Local $hSearch = FileFindFirstFile(@TempDir & "\wtf*.tmp")
	
	Local $strFile = FileFindNextFile($hSearch)
	While Not @error
		FileDelete(@TempDir & '\' & $strFile)
		$strFile = FileFindNextFile($hSearch)
	WEnd	
	
	FileClose($hSearch)
EndFunc ;==> ClearTemp

; Returns the file name from a path
Func FileGetName($strPath)
	Local $strBuffer, $strFileName, $strFileExt
	_PathSplit($strPath, $strBuffer, $strBuffer, $strFileName, $strFileExt)
	Return $strFileName & $strFileExt
EndFunc ;==> FileGetName

; Returns the file name from a path without the extension
Func FileGetNameOnly($strPath)
	Local $strBuffer, $strFileName
	_PathSplit($strPath, $strBuffer, $strBuffer, $strFileName, $strBuffer)
	Return $strFileName
EndFunc ;==> FileGetName

; Returns the style flags of a control
Func ControlGetStyle($title, $text, $controlID)
	Local $hWnd = ControlGetHandle($title, $text, $controlID)
	If @error Then
		Return SetError(1, 0, 0)
	EndIf
	
    Local $arrRet = DllCall("user32.dll", "int", "GetWindowLong", "hwnd", $hWnd, "int", -16) ; GWL_STYLE
    If @error Or Not $arrRet[0] Then
		Return SetError(1, 0, 0)
	Else
		SetError(0)
		Return $arrRet[0]
    EndIf
EndFunc ;==> ControlGetStyle

; RePost a WM_COMMAND message to a ctrl in a gui window
Func PostCommandMessage($title, $text, $idCommand)
	Local $hWnd = WinGetHandle($title, $text)
	If @error Then
		Return False
	EndIf
	
    Local $arrRet = DllCall("user32.dll", "int", "PostMessage", _
            "hwnd", $hWnd, _
            "int", 0x0111, _							; WM_COMMAND message
            "int", $idCommand, _						; The command to run
            "int", 0)									; Command from menu / accelerator
    If @error Or Not $arrRet[0] Then
		Return False
    EndIf
	
	Return True
EndFunc ;==> PostCommandMessage

; Tries to return the best name for the given application
Func GuessAppName($strAppPath)
	Local $strName
	
	$strName = FileGetVersion($strAppPath, "ProductName")
	If $strName And StringIsASCII($strName) And (StringLen($strName) < 35) Then
		Return $strName
	EndIf
	
	$strName = FileGetVersion($strAppPath, "FileDescription")
	If $strName And StringIsASCII($strName) And (StringLen($strName) < 35) Then
		Return $strName
	EndIf
	
	Return FileGetNameOnly($strAppPath)
EndFunc ; ==> GuessAppName

;;;;;;;;;;;;;;;;;;;;;;;;
; Guild Wars Autologin

; Load the command-line parameters
Func LoadCommandLines()
	If $CmdLine[0] And Not (FileGetVersion($strAppPath, "ProductName") == "Guild Wars") Then
		ShowError("The application you've set isn't Guild Wars." & @CRLF & "Command-line switches can only be used for Guild Wars.");
		Exit
	EndIf
	For $i = 1 To $CmdLine[0]
		For $j = 0 To $_sizeCommands - 1
			If StringLower(StringTrimLeft($CmdLine[$i], 1)) == $arrCommandLines[$j] And $i < $CmdLine[0] Then
				$arrLoginParams[$j] = $CmdLine[$i + 1]
				$bGotLoginParams = True
			EndIf
		Next
	Next
EndFunc ;==> LoadCommandLines

; Run the autologin process
Func AutoLogin()
	; Check if there are any command-lin
	If Not $bGotLoginParams Then
		Return
	EndIf
	
	; Wait until the Guild Wars main window becomes active and try to keep it active
	Local $strAppName = FileGetName($strAppPath)
	While WinWaitActive("[CLASS:ArenaNet_Dx_Window_Class]", "", $nGWWinTimeout) == 0
		; When timeouting, check if the process still exists, otherwise restart
		If Not ProcessExists($strAppName) Then
			main()
			Exit
		EndIf
	WEnd
	SendKeepActive("[CLASS:ArenaNet_Dx_Window_Class]")
	
	; Enter e-mail address
	If $arrLoginParams[$idEMail] <> "" Then
		Send("+{TAB}")
		Send($arrLoginParams[$idEMail])
		Send("{TAB}")
	EndIf
	
	; Enter password
	If $arrLoginParams[$idPassword] <> "" Then
		Send($arrLoginParams[$idPassword])
	EndIf

	; Enter character name
	If $arrLoginParams[$idCharacter] <> "" Then
		Send("{TAB}")
		Send($arrLoginParams[$idCharacter])
	EndIf
	
	; Automatically log in if password was given
	If $arrLoginParams[$idPassword] <> "" Then
		Send("{ENTER}")
	EndIf
EndFunc ;==> AutoLogin